import React, { useRef, useState } from 'react';
import PrimaryButton from '~/driven/utils/components/buttons/primary-button/PrimaryButton';
import { icons } from '~/driven/utils/constants/assertUrls';
import { staticMessages } from '~/driven/utils/constants/staticMessages';
import OtpAuthUsecase from '~/business-logic/generic/admin-user/authentication/otp-auth/usecase/otpAuthUsecase';
import PhonenumberAuthUsecase from '~/business-logic/generic/admin-user/authentication/phonnumber-auth/usecase/phonenumberAuthUsecase';
import StorageService from '~/driven/boundaries/storage-boundary';
import { appConfig } from '~/driven/utils/configs/appConfig';
import Notification from '~/driven/utils/components/Notification/Notification';
import OtpAuthUsecaseException from '~/business-logic/generic/admin-user/authentication/otp-auth/usecase/otpException';
import { AxiosError } from 'axios';
import { useUser } from '~/driven/utils/helpers/contexts/userContext';
import OtpCode from '../otp-code-inputs';
import PhoneNumberAuth from './PhoneNumberAuth';

interface IAuthenticationView {
  authPhone: PhonenumberAuthUsecase;
  otpAuth: OtpAuthUsecase;
}

/**
 * @todo should seperate logic and ui logic from the view
 */
export default function AuthenticationView(props: IAuthenticationView) {
  const { authPhone, otpAuth } = props;
  const [phoneNumberValue, setPhoneNumberValue] = useState('');
  const [error, setError] = useState('');
  const { setUser } = useUser();
  const otpChar = useRef<HTMLInputElement[]>([]);
  const statesName = {
    phonenumber: <PhoneNumberAuth stateData={{ setState: setPhoneNumberValue, stateValue: phoneNumberValue }} />,
    otp: <OtpCode ref={otpChar} />,
  };
  const [authState, setAuthState] = useState<keyof typeof statesName>('phonenumber');

  const submitForm = async (e: React.FormEvent) => {
    e.preventDefault();
    try {
      if (authState === 'phonenumber') {
        await authPhone.execute({ phonenumber: phoneNumberValue });
        setAuthState('otp');
      }
      if (authState === 'otp') {
        const otp = otpChar.current
          .map((inputItem) => {
            return inputItem.value;
          })
          .join('');
        const userModel = await otpAuth.execute({ otp, phonenumber: phoneNumberValue });
        const storage = StorageService.localStorage();
        storage.setData(appConfig.adminUserStorageKey, userModel);
        if (setUser) setUser(userModel);
      }
    } catch (errorException) {
      if (errorException instanceof OtpAuthUsecaseException) {
        setError(staticMessages.global.errors.otp);
      } else if (errorException instanceof AxiosError) {
        setError(errorException.response?.data?.description);
      } else if (errorException instanceof Error) {
        setError(errorException.message);
      }
    }
  };
  return (
    <div className='main-auth flex flex-nowrap justify-start flex-row-reverse h-screen w-screen'>
      {Boolean(error) && <Notification message={error} type='error' onCloseCallback={() => setError('')} />}
      <form
        onSubmit={submitForm}
        className='w-full px-7 md:px-20 md:w-[50%] lg:w-[35%]  min-w-[10rem] h-full shadow-lg shadow-slate-400 flex flex-col items-center justify-start pt-12'
      >
        <div className='w-48 h-[35%]'>
          <img src={icons.logoBlack} className='w-full h-12' alt='page icon' />
        </div>
        <div className='font-normal mb-4 text-lg self-start'>{staticMessages.global.enterPanel}</div>
        {statesName[authState]}
        <PrimaryButton
          onClick={() => null}
          title={staticMessages.global.submit}
          className='[background:var(--color-gradient-button)] hover:brightness-90 transition-all w-full h-11'
        />
      </form>
      <div className='hidden md:flex md:w-[50%] lg:w-[65%] h-full' />
    </div>
  );
}
